# 0006 - Firebase
**Updated:** 8 Apr 2023

## Status
<!-- Proposed | Accepted | Rejected | Deprecated | Superseded -->
Accepted

## Context
<!-- What is the issue that we’re seeing, that is motivating this decision or change -->
There are many different deployment options for web applications used within the MoonCat​Rescue ecosystem. Historically the MoonCat-related sites have been hosted on Heroku and GitLab Pages as simple mechanisms. But a more robust, unified platform would be better for long-term use.

## Decision
<!-- What is the change that we’re actually proposing or doing. -->
Use [Google Firebase](https://firebase.google.com/) as a hosting platform, utilizing its Hosting, Functions, and Cloud Firestore options to support the different MoonCat-related services.

### Firebase Storage
Firebase is a document-storage style database implementation. The billing model for Firebase is based on the number of documents read/written per billing cycle. Documents can be nested inside other documents (via a collection inside a document), however reading a document’s parent document counts as a separate read action. So, in general it is more cost-effective to structure the data inserted into Firebase as a smaller number of medium-sized documents, rather than a larger number of smaller-size documents, so this model should be the default data organizing approach applications should use, unless there’s a specific application benefit to using subcollections that outweighs the additional document read costs. 

### Event Storage
The Firestore storage platform provides rich querying options that can get lots of good information from a collection of blockchain Event data stored in Firebase. Getting that Event data into Firebase and keeping it updated is the key setup hurdle to benefiting from that. Most blockchain node implementations tackle this problem by having separate means of filling in archival data and catching up to current data, and that sort of model fits into the Firebase ecosystem too.

Filling in archival data is expected to be a slow process, and has large quantities of data that will need to be inserted. This action is not well-suited for a Firebase Function call (as that has a 5-minute max runtime requirement). So this shall be done by an external script, using the Admin SDK to authenticate to a Firebase instance. This means it will need to be triggered manually as part of application setup, but once done, should not need to be re-run unless data loss/corruption occurs. The archival storage script shall start at the earliest block Events could exist at, and progress forward to higher block heights from there. When a chunk of Events are successfully parsed, the script shall save the block height at the end of the chunk, and the rest of the application will trust that all block heights less than that point have been imported and verified.

Catching up to current data should ideally happen the moment new data is available. Once it’s known that new data exists, the catch-up action should be triggered, and the new Event data fetched and stored. Catching-up should be a relatively fast action, so having it be a Firebase Function is a good way to implement it. The catch-up action should work from most-recent (highest block height) and go backward in time (lower block heights) from there. The catch-up function should continue to work backwards until it runs out of time to process (5-minute max for Firebase Functions) or reaches the block that the archival data script indicated was the furthest it reached.

## Consequences
<!-- Outcomes, both positive and negative -->
For project maintainers, to deploy to Firebase, using [Firebase CLI](https://firebase.google.com/docs/cli) is needed. Most guidance for the tool assumes the developer will install Firebase CLI on their main workstation. With the MoonCat​Rescue projects using Docker for all development use, there’s a few nuances that need to be accounted for.

- The `firebase login` command directs the user to open a web link “on the same machine” as the running “login” process. It starts an OAuth process, and when it’s done, attempts to call back to the login process that starts listening on an open port. This does work from within Docker, unless the port it’s going to communicate on is opened. So instead, run `firebase login --no-localhost`. This modifies the process to have you copy/paste a final token to the service, to finish the login.
- The Firebase documentation for running within a CI/CD service [instructs the user](https://firebase.google.com/docs/cli#cli-ci-systems) to use the `firebase login:ci` command, to get a “refresh token” for the CI service to use. However, running that command gives a warning that the `login:ci` command is deprecated and the preferred way is “use a service account key with [`GOOGLE_APPLICATION_CREDENTIALS`](https://cloud.google.com/docs/authentication/application-default-credentials#GAC)”. That process is generally what GitLab recommends in [their documentation](https://docs.gitlab.com/ee/ci/cloud_services/google_cloud/); create an “identity pool”, have it trust just the CI/CD server (for GitLab, trusting the runner’s OpenID credentials), and have it hand back credentials for a service account that has permission to deploy the appropriate services. GitLab has created a [sample script](https://gitlab.com/guided-explorations/gcp/configure-openid-connect-in-gcp/-/blob/main/run_gcp_sts.sh) for how the CI job can use that process.
- A maintainer can do a manual deploy (using their own user credentials to Firebase), by doing `firebase login && firebase deploy` from within a `node:mooncat` (see [ADR0002](./0002-docker-development-environments.md)) container.

For developers, using the [Firebase Emulator tools](https://firebase.google.com/docs/emulator-suite) locally allows mocking in the Functions and Cloud Firestore capabilities. The maintainers of the Emulator Suite [have indicated](https://github.com/firebase/firebase-tools/issues/5197#issuecomment-1486034141) they consider the Emulator Suite a second-class tool for developers to use, and will instead focus on encouraging a workflow where developers use a separate Firebase project as a “pre-prod” environment. Having a separate pre-prod Firebase environment could be an additional cost to each developer if that project goes beyond the free tier level. Since the free tier only allows for 20,000 Firestore write actions per day, a MoonCatRescue project will very likely hit that limit as part of setup (even inserting a trait record for each of the 25,440 MoonCats would surpass that). That could be worked around since ingesting/setup scripts can do a chunk of data to start, and then get run the next day to pick up where they left off, but that’s not the greatest developer experience. So part of the effort of developing to work within the Firebase ecosystem might require contributing development to the Emulator Suite open source project, to keep the Emulator Suite working for this project’s needs.

The integration between GitLab’s CI/CD infrastructure and Firebase [is currently using](https://docs.gitlab.com/ee/ci/cloud_services/google_cloud/) a method that Google Cloud Platform has marked as deprecated. [An issue](https://gitlab.com/gitlab-org/gitlab/-/issues/394109) has been created in the main GitLab repository to update this. If the token-style authentication gets fully-removed from a version of Firebase Tools, before that issue is resolved, projects may need to stay on an older version of Firebase Tools to keep deployments working.