# 0010 - User-created Lists
**Updated:** 29 Jan 2024

## Status
<!-- Proposed | Accepted | Rejected | Deprecated | Superseded -->
Accepted

## Context
<!-- What is the issue that we’re seeing, that is motivating this decision or change -->
Many web applications have a need to create lists of things, have each user able to create their own lists, and have users be able to add/remove items from them. Wishlists, liked items, and bookmarks are common uses. Users may want to also create curated lists of arbitrary groupings, to share with others, or create multiple different public/private variants of their lists.

The typical "Web 2.0" way to implement that functionality is to have a centralized database which users log into for a specific application, and all actions and viewing of the lists need to happen through that centralized server. For the blockchain-based Web3 ecosystem, it would be better to devise a generic system that multiple sites can use, to remove the bottlenecks on data.

## Decision
<!-- What is the change that we’re actually proposing or doing. -->
The Kleros [Curate](https://curate.kleros.io/) platform was investigated as an option, but decided it was not a good fit for this use-case. Kleros Curate lists are lists designed to be curated by "the wisdom of the crowd", expecting many users to propose additions and removals from a few main lists. But that doesn't make sense for a list like "Alice's Wishlist of MoonCats to adopt someday" (only Alice is incentivized to keep that list updated, and only Alice knows the truthfulness of whether a MoonCat should be on that list or not).

The [Ethereum Attestation Service (EAS)](https://attest.sh/) is designed to make lightweight structured statements ("Attestations"), and the built-in EAS infrastructure gives some helpful defaults for all Attestations (a target/to address, able to be revoked, able to point to other Attestations as reference, etc.). Therefore it was chosen as the platform for storing user-created lists. The EAS infrastructure gives many options to make many different types of implementations, but for using it for the purpose of managing user-created lists, the following requirements must be met:

- Use Off-Chain Attestations. EAS can have on-chain Attestations, but the key benefit of having an Attestation that way is other smart contracts can access the data directly. Most use-cases for these sort of user lists are human-facing web interfaces, so don't need smart contracts to access them. 
- For assigning a token to a list:
    - Use the "to" address of the Attestation as the token collection the item(s) being added belong to
    - Use an EAS Schema with the following properties:
        - Token IDs: An array of `byte32` values. This value not being a `uint256` allows flexibility for different types of assets to be added to the user lists. If the "to" address is an [ERC721](https://eips.ethereum.org/EIPS/eip-721) or derivative contract, the `byte32` value will be interpreted as a `uint256` value.
        - Name: The name of the list. Stored as a `bytes32` value [for gas-efficiency](https://docs.attest.sh/docs/tutorials/gas-efficiency#choose-efficient-data-types) (List names are therefore limited to 32-characters or less)
- For describing a list:
    - Use the "to" address of the Attestation as the owner of the list being commented on
    - Use an EAS Schema with the following properties:
        - Name: The name of the list, prefixed by "Token List: "
        - Description: Details about the list
        - Rating: `uint8` numeric rating of the list. This is the smallest numeric field that EAS supports, which can hold integers from zero to 255. Common user experiences are to give users a "five-star rating" level of granularity, or just a boolean "like" option. To cover both these possibilities, values of 0-7 hold these meanings: the five middle values (2-6) shall represent one through five stars in rating. A value of 1 shall represent "off the scale bad" (e.g. not just "bad" but "malicious" or "fraudulent" or a "moderation needed" vote), and 7 shall represent a boolean "good" (e.g. "like" or "favorite"). A value of zero shall be used for if the user didn't give a rating (allowing for an Attesting to give a description of a List without ranking it). Values of 8-99 shall be reserved for future expansion of the standard. Applications wishing to create custom ratings with meanings specific to their application should use values 100-255 for their custom meanings.
- For each of these two types of Attestations, multiple schemas can be accepted as fulfilling these requirments. This allows for the project to evolve over time (if new schema features are desired, new schemas can be added while not losing the older Attestations), and lists generated by different web UIs to be able to be merged.

Even off-chain Attestations need to reference a specific chain, because if an item gets revoked, the revocation is stored on-chain. For the MoonCatRescue ecosystem, the Arbitrum blockchain will be used. Therefore, acceptable schemas for the two different list actions are:

- Assign Token to List:
    - Token List ([#171: `0x58de78ba175d13cb1699fdbc6c25a80ba2b21a943b5aac252ec367c878808d09`](https://arbitrum.easscan.org/schema/view/0x58de78ba175d13cb1699fdbc6c25a80ba2b21a943b5aac252ec367c878808d09))
- Describe a List:
    - Product Review ([#29: `0xeae0ff58b769be065ba9d54f4dd9b88c07de93a00e1e3b889a7099de02dd5549`](https://arbitrum.easscan.org/schema/view/0xeae0ff58b769be065ba9d54f4dd9b88c07de93a00e1e3b889a7099de02dd5549))

Assigning a token to a list shall use the following addresses as the "to" address to indicate different tokens in the MoonCatRescue ecosystem:

- MoonCats:
    - `0xf00e9cF6a96dFAd869f676022F49761632A3aA2C` (Primary) MoonCat traits contract on Arbitrum
    - `0x60cd862c9C687A9dE49aecdC3A99b74A4fc54aB6` MoonCatRescue Mainnet collection
    - `0xc3f733ca98e0dad0386979eb96fb1722a1a05e69` MoonCat Acclimator
- Lootprints
    - `0x450156f2d41CE63637A5Fc046a258A12A1DaE10B` (Primary) lootprints traits contract on Arbitrum
    - `0x1e9385eE28c5C7d33F3472f732Fb08CE3ceBce1F` lootprints Mainnet collection

For adding MoonCats to a List, the token identifier used shall be the MoonCat's Hex ID (`bytes5`). This will allow for Lunar and Hero MoonCats to be added to lists as well as Rescued.

The MoonCatRescue ecosystem will use IPFS for circulating off-chain Attestations (using it as a "Verifiable Data Ledger" as described in the [EAS documentation](https://docs.attest.sh/docs/core--concepts/onchain-vs-offchain#verifiable-data-ledger---a-third-option)). 

## Consequences
<!-- Outcomes, both positive and negative -->

The choice to use off-chain Attestations means if a user creates a list that is their "Wishlist" of items, that cannot be read by another smart contract directly to verify the user has wished for that. However, off-chain Attestations are [EIP712](https://eips.ethereum.org/EIPS/eip-712)-compatible. So, a smart contract could implement a `buyForUserWishlist` function by having the gift-giver give the Attestation as an input parameter. The smart contract can then parse it as an EIP712 message, and verify the user has actually wished for that item. That structure then requires the smart contract check the on-chain EAS registry to see if the off-chain Attestation [had been revoked](https://docs.attest.sh/docs/tutorials/revoking-attestations#revoking-off-chain-attestations)

The "add token to list" Attestation can define a list all-at-once, or incrementally. For most types of lists, it's assumed that adding items to the list would happen more frequently than removing items from the list. Using off-chain Attestations makes those frequently-occuring actions gas-less, and only revocations need gas to be on-chain. Choosing Arbitrum for the chain allows for strong proofs of Attestations (as a true rollup chain), while having lesser gas costs than Mainnet. 

The means for making sure IPFS distribution happens is not fully fleshed-out (should be a future ADR). As an initial step, some effort should be made to make sure off-chain Attestations are not lost, and that the MoonCatRescue host is not a bottleneck. Initially, this will take the form of storing the Attestations in the centralized MoonCatRescue Firestore database (with an appropriate API endpoint for accessing), and pinned to an Infura-hosted IPFS server. Longer-term, an IPFS ADR will be drafted to determine a plan for distributing IPFS pinning and querying to not just be the centralied server.
